#include <atmel_start.h>
#include <util/delay.h>
#include <stdio.h> //required for printf function
#include <avr/interrupt.h>


//--------------DEFINE DPV PARAMETERS HERE----------------------


#define REF_NEG 184					//Starting point for the sweep in DAC units
#define REF_POS 68					//End reference point for the sweep in DAC units
#define REF_INCREMENT 2				//Step increase in potential between pulses
#define PULSE 10					//Amplitude of each pulse
#define QUIET_TIME 450				//Quiet time between pulses in ms
#define PULSE_TIME 50				//Pulse time in ms
#define DPV_REPS 3					//Number of time to repeat DPV measurement 

#define THRESHOLD 60				//ADC units for the net current that indicates a positive sample - once this value is exceeded, LED on board lights up.
#define CAL_MODE 0x63				//Represents calibration mode (ASCII Character: lower case c)
#define DPV_MODE 0x64				//Represents DPV measurement mode (ASCII Character: lower case d)
#define PRECON_MODE	0x70			//Represents Preconditioning mode (ASCII Character: lower case p)

#define CONDITIONING_POT 48			//DAC value of conditioning potential - will depend upon cell calibration
#define CONDITIONING_TIME 30000		//Length of time conditioning takes place for in ms.


/*--------------DESCRIPTION OF FUNCTIONS-----------------------
The SimpleStat is currently configured to perform one of three routines under the control of the PC.  If only a DPV measurement is required, 
the code in DPV_Sweep() should be moved to main() and the remaider of the code below can be removed.  As soon as the battery is intserted, 
DPV sweeps will start and continue until power is removed.  This approach is useful for users who have a robust measurement system and only 
want to use the on/off lights to indicate the status of the sample.
*/

	
//Perform a DPV sweep the number of time defined in the DPV_REPS directive.
void DPV_Sweep(){
	
	uint8_t PULSE_NO = (REF_NEG - REF_POS)/REF_INCREMENT;
	uint8_t REF = 0;
	uint8_t i = 0;
	uint8_t j = 0;
	adc_result_t QUIET_CURR;
	adc_result_t PEAK_CURR;
	
	//Print output to UART terminal.
	printString("\n");
	printString("START\n");
	
	//Loop through DPV script the number of times defined in DPV_REPS to capture data
	for (j=0; j<DPV_REPS; j++)
	{		
		REF = REF_NEG;
		DAC_0_set_output(REF);
		_delay_ms(1000);
		
	//Perform the steps and pulses required for each sweep.	
	for (i=0;i<=PULSE_NO;i++) {
		_delay_ms(QUIET_TIME);
		QUIET_CURR = ADC_0_get_conversion(3);
		D_OUT_set_level(false);
		DAC_0_set_output(REF-PULSE);
		_delay_ms(PULSE_TIME);
		PEAK_CURR = ADC_0_get_conversion(3);
		REF = REF-REF_INCREMENT;
		DAC_0_set_output(REF);
		D_OUT_set_level(true);
		printByte(REF+REF_INCREMENT);
		printString("\t");
		printWord(QUIET_CURR);
		printString("\t");
		printWord(PEAK_CURR);
		printString("\t");
		printWord(PEAK_CURR-QUIET_CURR);
		printString("\n");
		
		
		if((PEAK_CURR-QUIET_CURR) > THRESHOLD)
			{
				D_OUT_set_level(true);		//turn LED on if defined threshold value is achieved.
			}
		}
	}
	DAC_0_set_output(127);
	printString("END\n");
}

//Condition electrode with a bias potential
void electrode_preconditioning(){
		printString("\nStarting cond\n");
		D_OUT_set_level(true);
		DAC_0_set_output(CONDITIONING_POT);
		_delay_ms(CONDITIONING_TIME);
		printString("\nEnd\n");
		D_OUT_set_level(false);
		DAC_0_set_output(127);
}

//Calibration mode
void calibrate_simpleStat(){
	printString("\nDAC output = REF POSITIVE\n");
	DAC_0_set_output(REF_POS);		//set DAC to min value
	receiveByte();					//wait for user to press button
	
	printString("DAC output = 127\n");
	DAC_0_set_output(127);			//set DAC to mid value
	receiveByte();					//wait for user to press button
	printString("DAC output = REF NEGATIVE\n");
	DAC_0_set_output(REF_NEG);	//set DAC to max value
	receiveByte();	
	DAC_0_set_output(127);

}

//Interrupt service routine - operates when the USART Rx data buffer receives a character.
//The behaviour of the SimpleStat depends upon the character pressed when under PC control.
ISR(USART0_RXC_vect){
	uint8_t readData;
	
	D_OUT_set_level(true);
	readData = receiveByte();  //Read data and echo back to clear Rx buffer and reset interrupt flag.
	_delay_ms(2000);
	D_OUT_set_level(false);
	if(readData == DPV_MODE){
		DPV_Sweep();	//Perform DPV sweeps.	
	}
	else if(readData == PRECON_MODE)
	{
		electrode_preconditioning();
	}
	else if(readData == CAL_MODE)
	{
		calibrate_simpleStat();
	}
	else
	{
		printString("Command not recognised.\n");
	}
}

//Initialize interrupt routine
void initInterruptUSART( void)
{ USART0.CTRLA |= (1 << USART_RXCIE_bp); /* enable USART0 receive data ISR */
	sei(); /* set (global) interrupt enable bit */
}

//Main routine used when SimpleStat is under PC control.
int main(void)
{

	/* Initializes MCU, drivers and middleware */
	atmel_start_init();
	initInterruptUSART();
	DAC_0_set_output(127);
	while(1){
		printString("\nPress c for cal, d for DPV p for precond\n");
		printString("END\n");
		_delay_ms(5000);
	}
	
	
}